@@ -7,8 +7,6 @@ use nu_protocol::{
     Config, PipelineData, Value,
 };
 use reedline::Prompt;
-use std::borrow::Cow;
-use std::sync::Arc;
 
 // Name of environment variable where the prompt could be stored
 pub(crate) const PROMPT_COMMAND: &str = "PROMPT_COMMAND";
@@ -28,8 +26,8 @@ pub(crate) const TRANSIENT_PROMPT_MULTILINE_INDICATOR: &str =
     "TRANSIENT_PROMPT_MULTILINE_INDICATOR";
 // According to Daniel Imms @Tyriar, we need to do these this way:
 // <133 A><prompt><133 B><command><133 C><command output>
-const PRE_PROMPT_MARKER: &str = "\x1b]133;A\x1b\\";
-const POST_PROMPT_MARKER: &str = "\x1b]133;B\x1b\\";
+pub(crate) const PRE_PROMPT_MARKER: &str = "\x1b]133;A\x1b\\";
+pub(crate) const POST_PROMPT_MARKER: &str = "\x1b]133;B\x1b\\";
 
 fn get_prompt_string(
     prompt: &str,
@@ -98,12 +96,12 @@ fn get_prompt_string(
         })
 }
 
-pub(crate) fn update_prompt<'prompt>(
+pub(crate) fn update_prompt(
     config: &Config,
     engine_state: &EngineState,
     stack: &Stack,
-    nu_prompt: &'prompt mut NushellPrompt,
-) -> &'prompt dyn Prompt {
+    nu_prompt: &mut NushellPrompt,
+) {
     let mut stack = stack.clone();
 
     let left_prompt_string = get_prompt_string(PROMPT_COMMAND, config, engine_state, &mut stack);
@@ -146,125 +144,55 @@ pub(crate) fn update_prompt<'prompt>(
         (prompt_vi_insert_string, prompt_vi_normal_string),
         config.render_right_prompt_on_last_line,
     );
-
-    let ret_val = nu_prompt as &dyn Prompt;
     trace!("update_prompt {}:{}:{}", file!(), line!(), column!());
-
-    ret_val
 }
 
-struct TransientPrompt {
-    engine_state: Arc<EngineState>,
-    stack: Stack,
-}
-
-/// Try getting `$env.TRANSIENT_PROMPT_<X>`, and get `$env.PROMPT_<X>` if that fails
-fn get_transient_prompt_string(
-    transient_prompt: &str,
-    prompt: &str,
+/// Construct the transient prompt based on the normal nu_prompt
+pub(crate) fn make_transient_prompt(
     config: &Config,
     engine_state: &EngineState,
     stack: &mut Stack,
-) -> Option<String> {
-    get_prompt_string(transient_prompt, config, engine_state, stack)
-        .or_else(|| get_prompt_string(prompt, config, engine_state, stack))
-}
+    nu_prompt: &NushellPrompt,
+) -> Box<dyn Prompt> {
+    let mut nu_prompt = nu_prompt.clone();
 
-impl Prompt for TransientPrompt {
-    fn render_prompt_left(&self) -> Cow<str> {
-        let mut nu_prompt = NushellPrompt::new();
-        let config = &self.engine_state.get_config().clone();
-        let mut stack = self.stack.clone();
-        nu_prompt.update_prompt_left(get_transient_prompt_string(
-            TRANSIENT_PROMPT_COMMAND,
-            PROMPT_COMMAND,
-            config,
-            &self.engine_state,
-            &mut stack,
-        ));
-        nu_prompt.render_prompt_left().to_string().into()
+    if let Some(s) = get_prompt_string(TRANSIENT_PROMPT_COMMAND, config, engine_state, stack) {
+        nu_prompt.update_prompt_left(Some(s))
     }
 
-    fn render_prompt_right(&self) -> Cow<str> {
-        let mut nu_prompt = NushellPrompt::new();
-        let config = &self.engine_state.get_config().clone();
-        let mut stack = self.stack.clone();
-        nu_prompt.update_prompt_right(
-            get_transient_prompt_string(
-                TRANSIENT_PROMPT_COMMAND_RIGHT,
-                PROMPT_COMMAND_RIGHT,
-                config,
-                &self.engine_state,
-                &mut stack,
-            ),
-            config.render_right_prompt_on_last_line,
-        );
-        nu_prompt.render_prompt_right().to_string().into()
+    if let Some(s) = get_prompt_string(TRANSIENT_PROMPT_COMMAND_RIGHT, config, engine_state, stack)
+    {
+        nu_prompt.update_prompt_right(Some(s), config.render_right_prompt_on_last_line)
     }
 
-    fn render_prompt_indicator(&self, prompt_mode: reedline::PromptEditMode) -> Cow<str> {
-        let mut nu_prompt = NushellPrompt::new();
-        let config = &self.engine_state.get_config().clone();
-        let mut stack = self.stack.clone();
-        nu_prompt.update_prompt_indicator(get_transient_prompt_string(
-            TRANSIENT_PROMPT_INDICATOR,
-            PROMPT_INDICATOR,
-            config,
-            &self.engine_state,
-            &mut stack,
-        ));
-        nu_prompt.update_prompt_vi_insert(get_transient_prompt_string(
-            TRANSIENT_PROMPT_INDICATOR_VI_INSERT,
-            PROMPT_INDICATOR_VI_INSERT,
-            config,
-            &self.engine_state,
-            &mut stack,
-        ));
-        nu_prompt.update_prompt_vi_normal(get_transient_prompt_string(
-            TRANSIENT_PROMPT_INDICATOR_VI_NORMAL,
-            PROMPT_INDICATOR_VI_NORMAL,
-            config,
-            &self.engine_state,
-            &mut stack,
-        ));
-        nu_prompt
-            .render_prompt_indicator(prompt_mode)
-            .to_string()
-            .into()
+    if let Some(s) = get_prompt_string(TRANSIENT_PROMPT_INDICATOR, config, engine_state, stack) {
+        nu_prompt.update_prompt_indicator(Some(s))
     }
-
-    fn render_prompt_multiline_indicator(&self) -> Cow<str> {
-        let mut nu_prompt = NushellPrompt::new();
-        let config = &self.engine_state.get_config().clone();
-        let mut stack = self.stack.clone();
-        nu_prompt.update_prompt_multiline(get_transient_prompt_string(
-            TRANSIENT_PROMPT_MULTILINE_INDICATOR,
-            PROMPT_MULTILINE_INDICATOR,
-            config,
-            &self.engine_state,
-            &mut stack,
-        ));
-        nu_prompt
-            .render_prompt_multiline_indicator()
-            .to_string()
-            .into()
-    }
-
-    fn render_prompt_history_search_indicator(
-        &self,
-        history_search: reedline::PromptHistorySearch,
-    ) -> Cow<str> {
-        NushellPrompt::new()
-            .render_prompt_history_search_indicator(history_search)
-            .to_string()
-            .into()
-    }
-}
-
-/// Construct the transient prompt
-pub(crate) fn transient_prompt(engine_state: Arc<EngineState>, stack: &Stack) -> Box<dyn Prompt> {
-    Box::new(TransientPrompt {
+    if let Some(s) = get_prompt_string(
+        TRANSIENT_PROMPT_INDICATOR_VI_INSERT,
+        config,
         engine_state,
-        stack: stack.clone(),
-    })
+        stack,
+    ) {
+        nu_prompt.update_prompt_vi_insert(Some(s))
+    }
+    if let Some(s) = get_prompt_string(
+        TRANSIENT_PROMPT_INDICATOR_VI_NORMAL,
+        config,
+        engine_state,
+        stack,
+    ) {
+        nu_prompt.update_prompt_vi_normal(Some(s))
+    }
+
+    if let Some(s) = get_prompt_string(
+        TRANSIENT_PROMPT_MULTILINE_INDICATOR,
+        config,
+        engine_state,
+        stack,
+    ) {
+        nu_prompt.update_prompt_multiline(Some(s))
+    }
+
+    Box::new(nu_prompt)
 }
