@@ -2,6 +2,7 @@ mod eager;
 mod expressions;
 mod lazy;
 mod series;
+mod stub;
 mod utils;
 mod values;
 
@@ -15,6 +16,7 @@ use nu_protocol::engine::{EngineState, StateWorkingSet};
 pub fn add_dataframe_context(mut engine_state: EngineState) -> EngineState {
     let delta = {
         let mut working_set = StateWorkingSet::new(&engine_state);
+        working_set.add_decl(Box::new(stub::Dfr));
         add_series_decls(&mut working_set);
         add_eager_decls(&mut working_set);
         add_expressions(&mut working_set);
