@@ -33,10 +33,13 @@ impl Command for AllFalse {
                 description: "Returns true if all values are false",
                 example: "[false false false] | dfr into-df | dfr all-false",
                 result: Some(
-                    NuDataFrame::try_from_columns(vec![Column::new(
-                        "all_false".to_string(),
-                        vec![Value::test_bool(true)],
-                    )])
+                    NuDataFrame::try_from_columns(
+                        vec![Column::new(
+                            "all_false".to_string(),
+                            vec![Value::test_bool(true)],
+                        )],
+                        None,
+                    )
                     .expect("simple df for test should not fail")
                     .into_value(Span::test_data()),
                 ),
@@ -47,10 +50,13 @@ impl Command for AllFalse {
     let res = ($s > 9);
     $res | dfr all-false"#,
                 result: Some(
-                    NuDataFrame::try_from_columns(vec![Column::new(
-                        "all_false".to_string(),
-                        vec![Value::test_bool(false)],
-                    )])
+                    NuDataFrame::try_from_columns(
+                        vec![Column::new(
+                            "all_false".to_string(),
+                            vec![Value::test_bool(false)],
+                        )],
+                        None,
+                    )
                     .expect("simple df for test should not fail")
                     .into_value(Span::test_data()),
                 ),
@@ -88,8 +94,11 @@ fn command(
 
     let value = Value::bool(!bool.any(), call.head);
 
-    NuDataFrame::try_from_columns(vec![Column::new("all_false".to_string(), vec![value])])
-        .map(|df| PipelineData::Value(NuDataFrame::into_value(df, call.head), None))
+    NuDataFrame::try_from_columns(
+        vec![Column::new("all_false".to_string(), vec![value])],
+        None,
+    )
+    .map(|df| PipelineData::Value(NuDataFrame::into_value(df, call.head), None))
 }
 
 #[cfg(test)]
